#define _GNU_SOURCE

#include "gettext.h"
#include "print_error.h"
#include "replace_string.h"
#include "struct_CPInfo.h"
#include "struct_OPArg.h"
#include "write_log_hash_check.h"

#include <unistd.h>

/* 関数プロトタイプ */
void verify_md5sum(const OPArg *oparg, const SDir *sdir, CPInfo *cpinfo, CPDD *cpdd);

/*******************************************************************************
*******************************************************************************/
void verify_md5sum(const OPArg *oparg, const SDir *sdir, CPInfo *cpinfo, CPDD *cpdd)
{
	/*
	 * 'でくくり、'を'\''に置換するようにしたので、
	 * 置換後の文字列の長さを最大PATH_MAXの四倍と仮定。
	 */
	const int REPLACE_LEN = PATH_MAX * 4;
	FILE *pipe_dst = NULL;
	char pipe_buf_dst[PATH_MAX];
	char re_dst[REPLACE_LEN];
	char command_dst[REPLACE_LEN + sizeof("md5sum -- \'\'") + 1];
	char *ddst = NULL;
	int read_size = 0;

	pipe_buf_dst[0] = '\0';
	re_dst[0] = '\0';
	command_dst[0] = '\0';

	strcpy(re_dst, cpinfo->dst);

	/*
	 * 特殊文字の置換
	 * bashのみ対応、他は未確認。
	 * かならず最初に\を置換する。
	*/
/*
	replace_string(re_dst, "\\", "\\\\", REPLACE_LEN);
	replace_string(re_dst, "\"", "\\\"", REPLACE_LEN);
	replace_string(re_dst, "`", "\\`", REPLACE_LEN);
	replace_string(re_dst, "$", "\\$", REPLACE_LEN);
	sprintf(command_dst, "%s%s%s%s", "md5sum -- ", "\"", re_dst, "\"");
*/
	/*
	 * 「verify_md5sum.c」の特殊文字の置換は
	 * ファイル名中の「'」を「'\''」に置換しておいて
	 * 「md5sum ./'ファイル名'」とするのが無難かと
	 * 匿名さんによるプロジェクトレビュー - 2012-01-15 00:56
	 * 
	 * ↑サンクス！
	 */
	replace_string(re_dst, "'", "'\\''", REPLACE_LEN);
	sprintf(command_dst, "%s%s%s%s", "md5sum -- ", "\'", re_dst, "\'");

	{
		errno = 0;
		/* バッファ掃除用の領域 */
		char ctmp[PATH_MAX];

		pipe_dst = popen(command_dst, "r");

		if(pipe_dst != NULL)
		{
			read_size = fread(pipe_buf_dst, 1, PATH_MAX, pipe_dst);
			/* パイプのバッファを空にするためにグルグル回す */
			while(fread(ctmp, 1, PATH_MAX, pipe_dst) > 0) {}
		}
	}

	if(pipe_dst != NULL)
	{
		/* チェックしとかないと (半角スペース)が含まれていない場合、SIGSEGVで落ちる */
		if(strstr(pipe_buf_dst, " "))
		{
			ddst = strtok(pipe_buf_dst, " ");

			/*
			 * Ubuntu 10.10で、ファイル名に\が含まれているファイルをsha1sumやmd5sumに渡すと、
			 * 何故かハッシュ値の前に\が挿入されるっぽい。バグか？
			 * その所為で文字列の長さが33や41になってハッシュ値が不一致と判定されるので、
			 * 文字列の先頭が\だった場合、文字列を一つ前にずらす。
			 */
			if(ddst[0] == '\\')
			{
				memmove(ddst, ddst + 1, strlen(ddst));
			}

			/* MD5 == 32桁　SHA-1 == 40桁 */
			if((strcmp(cpinfo->src_md5, ddst) == 0) && ((strlen(cpinfo->src_md5) == 32) || (strlen(cpinfo->src_md5) == 40)))
			{
				cpinfo->hash_check = true;

				if(oparg->V == VERBOS)
				{
					if(oparg->M == NOT)
					{
						printf(_("%s : %s\n"), cpinfo->src_md5, cpinfo->src);
					}

					printf(_("%s : %s\n"), ddst, cpinfo->dst);
					fflush(stdout);
				}

				if(oparg->L == LOG)
				{
					write_log_hash_check(cpinfo, oparg, cpinfo->src_md5, ddst, sdir, cpdd, true);
				}
			}
			else
			{
				print_error("NULL", cpinfo->dst, __FILE__, __LINE__, cpdd);
				write_log_hash_check(cpinfo, oparg, cpinfo->src_md5, ddst, sdir, cpdd, false);
				cpinfo->dst_para = DELETE;

				errno = 0;
				if(unlink(cpinfo->dst) == -1)
				{
					if(errno != ENOENT)
					{
						print_error("unlink", cpinfo->dst, __FILE__, __LINE__, cpdd);
					}
				}
				else
				{
					fprintf(stderr, _("remove : %s\n"), cpinfo->dst);
				}
			}
		}
		else
		{
			print_error("NULL", cpinfo->dst, __FILE__, __LINE__, cpdd);
			write_log_hash_check(cpinfo, oparg, "NULL", "NULL", sdir, cpdd, false);
			cpinfo->dst_para = INITIAL;
		}
	}
	else
	{
		print_error("NULL", cpinfo->dst, __FILE__, __LINE__, cpdd);
		write_log_hash_check(cpinfo, oparg, "NULL", "NULL", sdir, cpdd, false);
		cpinfo->dst_para = INITIAL;
	}

	/* 念のため、pipe詰まり対策 */
	for(;;)
	{
		read_size = 0;
		read_size += fread(pipe_buf_dst, 1, PATH_MAX, pipe_dst);

		if(read_size <= 0)
		{
			break;
		}
	}

	if(pipe_dst != NULL)
	{
		pclose(pipe_dst);
	}
}
